/*  euler.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "euler.h"
#include "../parameter.h"
#include "../constant.h"
#include "../matrix.h"
#include "../tmatrix.h"
#include "../vartype.h"
#include <kdebug.h>

MethodEuler::MethodEuler(void) : MapInternal("MEuler",0)
{
  m_nParameters=5;
  m_parameters=new Parameter *[m_nParameters];

  m_parameters[0]=new Parameter("f");
  m_parameters[1]=new Parameter("x");
  m_parameters[1]->setDataType("Variable");
  m_parameters[2]=new Parameter("a");
  m_parameters[2]->setDataType("Constant");
  m_parameters[3]=new Parameter("b");
  m_parameters[3]->setDataType("Constant");
  m_parameters[4]=new Parameter("h");
  m_parameters[4]->setDataType("Constant");
}

MethodEuler::MethodEuler(const MethodEuler &m) : MapInternal(m)
{
}

MethodEuler::~MethodEuler()
{
}
  
Map *MethodEuler::copy(void) const
{
  return new MethodEuler(*this);
}
  
Map *MethodEuler::eval(const QDict <Map> &vars)
{
  kdDebug() << "Euler::eval\n" << endl;
  QDict <Map> *vars2=evalParams(vars);
  if (!checkParamTypes(*vars2)) return 0L;
  Map *f=vars2->find("f");
  kdDebug() << f->string() << endl;
  Map *xvar=vars2->find("x");
  kdDebug() << QString("x -------------------------> %1").arg(xvar->string()) << endl;
  T a(*vars2->find("a")->t());
  T b(*vars2->find("b")->t());
  T h(*vars2->find("h")->t());
  Map *tx;
  QDict <Map> tmpvars;

  kdDebug() << "Euler::eval (" << ((double)b-(double)a)/(double)h << ")\n" << endl;
  TMatrix *m=new TMatrix(2,((double)b-(double)a)/(double)h);
  kdDebug() << "Euler::eval2\n" << endl;
  double x=a;
  int i=0;
  while (x<(double)b)
  {
//    kdDebug() << "i=" << i << " (" << m->nCols() << "), x=" << x << "\n" << endl;
    m->set(0,i,tx=new Constant(x));
    tmpvars.replace(xvar->name(),tx); // XXX necesita que f dependa de x
    m->set(1,i++,f->eval(tmpvars));
    x+=(double)h;
  }
  kdDebug() << "Euler::eval1(end)\n" << endl;
  delete vars2;
  kdDebug() << "Euler::eval(end)\n" << endl;
  
  return new Matrix(m);
}

QString MethodEuler::string(void) const
{
  return "MethodEuler";
}
